#
from enigma import *
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.ChoiceBox import ChoiceBox
from Components.ActionMap import ActionMap, NumberActionMap
from Components.ScrollLabel import ScrollLabel
from Components.GUIComponent import *
from Components.MenuList import MenuList
from Components.Input import Input
from Screens.Console import Console
from Plugins.Plugin import PluginDescriptor
from Screens.ImageWizard import ImageWizard
from Plugins.Plugin import PluginDescriptor

import os

###############################################################################
shell_pluginversion = "V3.0"
shell_script = "shell.sh"
shell_readme = "readme.txt"
shell_commands = "commands.lst"
shell_keymaps = "keymaps.lst"
shell_pluginpath = "/usr/lib/enigma2/python/Plugins/Extensions/Shell"

###############################################################################
def main(session,**kwargs):
    try:    
     	session.open(Shell)
    except:
        print "[SHELL] Pluginexecution failed"
###############################################################################
def autostart(reason,**kwargs):
    if reason == 0:
        print "[SHELL] no autostart"
###############################################################################
def Plugins(**kwargs):
    return PluginDescriptor(
        name="Shell", 
        description="plugin to execute Shell commands", 
        where = PluginDescriptor.WHERE_PLUGINMENU,
        fnc = main
        )
###############################################################################
###############################################################################
class Shell(Screen):
    skin = """
        <screen position="100,100" size="500,400" title="Shell Plugin Menu" >
            <widget name="menu" position="10,10" size="490,390" scrollbarMode="showOnDemand" />
        </screen>"""
        
    def __init__(self, session, args = 0):
        self.skin = Shell.skin
        self.session = session
        Screen.__init__(self, session)
        self.menu = args
        list = []
        list.append((_("execute shell command"), "shellwizzard"))
        list.append((_("execute shell command unconfimed"), "unconfirmedwizzard"))
        list.append((_("execute user defined commands"), "userwizzard"))
        list.append((_("execute system commands"), "systemwizzard"))
        list.append((_("execute archive commands"), "archivewizzard"))
        list.append((_("execute directory commands"), "filewizzard"))
        list.append((_("execute disk commands"), "diskwizzard"))
        list.append((_("enable  alternative Keymap"), "alternatekeys"))        
        list.append((_("dsaible alternative Keymap"), "standardkeys"))        
        list.append((_("exit shell resets working directory to /"), "exit"))        
        list.append((_("help"), "help"))        
        list.append((_("about"), "about"))        
        self["menu"] = MenuList(list)
        self["actions"] = ActionMap(["WizardActions", "DirectionActions"],{"ok": self.go,"back": self.close,}, -1)
        
    def go(self):
        returnValue = self["menu"].l.getCurrentSelection()[1]
        if returnValue is not None:
           if returnValue is "shellwizzard":
               ShellWizzard(self.session)
           elif returnValue is "systemwizzard":
               SystemWizzard(self.session)
           elif returnValue is "unconfirmedwizzard":
               UnconfirmedWizzard(self.session)
           elif returnValue is "userwizzard":
               UserWizzard(self.session)
           elif returnValue is "filewizzard":
               FileWizzard(self.session)
           elif returnValue is "diskwizzard":
               DiskWizzard(self.session)
           elif returnValue is "archivewizzard":
               ArchiveWizzard(self.session)
           elif returnValue is "alternatekeys":
               KeyWizzard(self.session)
           elif returnValue is "standardkeys":
               title = "enabling standard keymap.xml"
               cmd = "%s/%s standardkeys" % (shell_pluginpath,shell_script)
               self.session.open(Console,_(title),[cmd])
           elif returnValue is "exit":
               title = "exit resets working directory to /"
               cmd = "%s/%s exit" % (shell_pluginpath,shell_script)
               self.session.open(Console,_(title),[cmd])
           elif returnValue is "help":
               title = "help shows readme.txt"
               cmd = "cat %s/%s" % (shell_pluginpath,shell_readme)
               self.session.open(Console,_(title),[cmd])
           elif returnValue is "about":
               self.session.open(MessageBox,_("Shell Enigma2 Plugin %s by thowi\n\nShell execution plugin\n\nEnjoy executing Shell commands with your remote control !" % shell_pluginversion), MessageBox.TYPE_INFO)

###############################################################################
###############################################################################

class ShellWizzard(Screen):
    def __init__(self, session):

        self.session = session
        self.askForShellCommand()

    def askForShellCommand(self):
	       self.session.openWithCallback(self.processingShellCommand,InputBox, title=_("Enter Shell command for execution"), text="ls", maxSize=False, type=Input.TEXT)

    def processingShellCommand(self,commandname):
        if commandname is None:
            self.skipShell("No shell command passed, skipping execution ")
        else:
            self.commandname = commandname
            self.session.openWithCallback(self.ShellExecution,MessageBox,_("are you sure to execute Shell command %s ?" % (self.commandname)), MessageBox.TYPE_YESNO)

    def ShellExecution(self,answer):
        if answer is None:
            self.skipShell("answer is None")
        if answer is False:
            self.skipShell("you were not confirming")
        else:
            title = "Executing Shell comand %s" %(self.commandname)
            cmd = "%s/%s %s" % (shell_pluginpath,shell_script,self.commandname)
            self.session.open(Console,_(title),[cmd])
            
    def skipShell(self,reason):
        self.session.open(MessageBox,_("Saving Shell was canceled, because %s" % reason), MessageBox.TYPE_WARNING)

###############################################################################
###############################################################################

class UnconfirmedWizzard(Screen):
    def __init__(self, session):

        self.session = session
        self.askForUnconfirmedCommand()

    def askForUnconfirmedCommand(self):
	       self.session.openWithCallback(self.processingUnconfirmedCommand,InputBox, title=_("Enter Shell command for execution"), text="ls", maxSize=False, type=Input.TEXT)

    def processingUnconfirmedCommand(self,commandname):
        if commandname is None:
            self.skipUnconfirmed("No shell command passed, skipping execution ")
        else:
            self.commandname = commandname
            title = "Executing Shell comand %s unconfirmed" %(self.commandname)
            cmd = "%s/%s %s" % (shell_pluginpath,shell_script,self.commandname)
            self.session.open(Console,_(title),[cmd])
            
    def skipUnconfirmed(self,reason):
        self.session.open(MessageBox,_("Saving Shell was canceled, because %s" % reason), MessageBox.TYPE_WARNING)

###############################################################################
###############################################################################

class SystemWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForSystemCommand()
        
    def askForSystemCommand(self):
            self.session.openWithCallback(self.askForAllSystemCommand,ChoiceBox,_("select system command to be executed"),self.getSystemCommandList())

    def askForAllSystemCommand(self,source):
        if source is None:
            self.skipSystemCommand("no system command passed, skipping execution")
        else:
            self.source = source [1]
            self.session.openWithCallback(self.systemCommand,MessageBox,_("are you sure to execute %s" % self.source), MessageBox.TYPE_YESNO)
            
    def systemCommand(self,answer):
        if answer is None:
            self.skipSystemCommand("answer is None")
        if answer is False:
            self.skipSystemCommand("you were not confirming")
        else:
            title = "executing system command on Dreambox"
            cmd = "%s/%s %s"  % (shell_pluginpath,shell_script,self.source)
            self.session.open(Console,_(title),[cmd])
            
    def skipSystemCommand(self,reason):
        self.session.open(MessageBox,_("executing system command was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getSystemCommandList(self):
        command = []
        command.append((_("shutdown -r now"),"shutdown -r now"))
        command.append((_("shutdown -r -F now"),"shutdown -r -F now"))
        command.append((_("shutdown -h now"),"shutdon -h now"))
        command.append((_("killall enigma2"),"killall enigma2"))
        command.append((_("ps ax"),"ps ax"))
        command.append((_("netstat"),"netstat"))
        command.append((_("ifconfig eth0"),"ifconfig eth0"))
        command.append((_("ping 192.168.0.1"),"ping 192.168.0.1"))
        return command


###############################################################################

class FileWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForFileCommand()
        
    def askForFileCommand(self):
            self.session.openWithCallback(self.askForAllFileCommand,ChoiceBox,_("select directory command to be executed"),self.getFileCommandList())

    def askForAllFileCommand(self,source):
        if source is None:
            self.skipFileCommand("no directory command passed, skipping execution")
        else:
            self.source = source [1]
            self.session.openWithCallback(self.FileCommand,MessageBox,_("are you sure to %s" % self.source), MessageBox.TYPE_YESNO)
            
    def FileCommand(self,answer):
        if answer is None:
            self.skipFileCommand("answer is None")
        if answer is False:
            self.skipFileCommand("you were not confirming")
        else:
            title = "executing directory command on Dreambox"
            cmd = "%s/%s %s"  % (shell_pluginpath,shell_script,self.source)
            self.session.open(Console,_(title),[cmd])
            
    def skipFileCommand(self,reason):
        self.session.open(MessageBox,_("executing directory command was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getFileCommandList(self):
        command = []
        command.append((_("ls -alh"),"ls -alh"))
#        command.append((_("exit"),"exit"))
        command.append((_("pwd"),"pwd"))
        command.append((_("du -h /"),"du -h /"))
        command.append((_("cd /var/tmp"),"cd /var/tmp"))
        command.append((_("cd /media/hdd"),"cd /media/hdd"))
        command.append((_("cd /media/hdd/movie"),"cd /media/hdd/movie"))
        command.append((_("cd /media/cf"),"cd /media/cf"))
        command.append((_("cd /media/usb"),"cd /media/usb"))
        command.append((_("cd /media/mb"),"cd /media/mb"))
        command.append((_("cd /usr/lib/enigma2/python/Plugins"),"cd /usr/lib/enigma2/python/Plugins"))
        command.append((_("cd /dev/ide/host0/bus0/target0/lun0"),"cd /dev/ide/host0/bus0/target0/lun0"))
        command.append((_("cd /dev/ide/host1/bus0/target0/lun0"),"cd /dev/ide/host1/bus0/target0/lun0"))
        command.append((_("cd /usr/script"),"cd /usr/script"))
        return command

###############################################################################



###############################################################################

class DiskWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForDiskCommand()
        
    def askForDiskCommand(self):
            self.session.openWithCallback(self.askForCommand,ChoiceBox,_("select disk command to be executed"),self.getDiskCommandList())

    def askForCommand(self,source):
        if source is None:
            self.skipDiskCommand("no disk command passed, skipping execution")
        else:
            self.source = source [1]
            self.session.openWithCallback(self.systemCommand,MessageBox,_("are you sure to %s" % self.source), MessageBox.TYPE_YESNO)
            
    def systemCommand(self,answer):
        if answer is None:
            self.skipDiskCommand("answer is None")
        if answer is False:
            self.skipDiskCommand("you were not confirming")
        else:
            title = "executing disk command on Dreambox"
            cmd = "%s/%s %s"  % (shell_pluginpath,shell_script,self.source)
            self.session.open(Console,_(title),[cmd])
            
    def skipDiskCommand(self,reason):
        self.session.open(MessageBox,_("executing disk command was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getDiskCommandList(self):
        command = []
        command.append((_("diskdoctor.sh"),"/usr/lib/enigma2/python/Plugins/Extensions/Shell/diskdoctor.sh"))
        command.append((_("df -h"),"df -h"))
        command.append((_("mount"),"mount"))
        command.append((_("mount -a"),"mount -a"))
        command.append((_("cat /etc/fstab"),"cat /etc/fstab"))
	command.append((_("ls /dev/ide/host0/bus0/target0/lun0"),"ls /dev/ide/host0/bus0/target0/lun0"))
	command.append((_("ls /dev/ide/host1/bus0/target0/lun0"),"ls /dev/ide/host1/bus0/target0/lun0"))
        command.append((_("fdisk -l /dev/ide/host0/bus0/target0/lun0/disc"),"fdisk -l /dev/ide/host0/bus0/target0/lun0/disc"))
        command.append((_("fdisk -l /dev/ide/host1/bus0/target0/lun0/disc"),"fdisk -l /dev/ide/host1/bus0/target0/lun0/disc"))
        command.append((_("fsck.ext3 -p /dev/ide/host0/bus0/target0/lun0/part1"),"fsck.ext3 -p /dev/ide/host0/bus0/target0/lun0/part1"))
        command.append((_("sed -ie s!/dev/ide/host0!#/dev/ide/host0!g /etc/fstab"),"sed -ie s!/dev/ide/host0!#/dev/ide/host0!g /etc/fstab"))
        command.append((_("sed -ie s!#/dev/ide/host0!/dev/ide/host0!g /etc/fstab"),"sed -ie s!#/dev/ide/host0!/dev/ide/host0!g /etc/fstab"))
        return command

###############################################################################
###############################################################################

class ArchiveWizzard(Screen):
    def __init__(self, session):
        self.session = session
        self.askForArchiveCommand()
        
    def askForArchiveCommand(self):
            self.session.openWithCallback(self.askForCommand,ChoiceBox,_("select archive command to be executed"),self.getArchiveCommandList())

    def askForCommand(self,source):
        if source is None:
            self.skipArchiveCommand("no archive command passed, skipping execution")
        else:
            self.source = source [1]
            self.session.openWithCallback(self.systemCommand,MessageBox,_("are you sure to %s" % self.source), MessageBox.TYPE_YESNO)
            
    def systemCommand(self,answer):
        if answer is None:
            self.skipArchiveCommand("answer is None")
        if answer is False:
            self.skipArchiveCommand("you were not confirming")
        else:
            title = "executing archive command on Dreambox"
            cmd = "%s/%s %s"  % (shell_pluginpath,shell_script,self.source)
            self.session.open(Console,_(title),[cmd])
            
    def skipArchiveCommand(self,reason):
        self.session.open(MessageBox,_("executing archive command was canceled, because %s" % reason), MessageBox.TYPE_WARNING)
        
    def getArchiveCommandList(self):
        command = []
        command.append((_("bunzip2 /var/tmp/*.bz2"),"bunzip2 /var/tmp/*.bz2"))
        command.append((_("gzip -d /var/tmp/*.gz"),"gzip -d /var/tmp/*.gz"))
        command.append((_("tar -tvf /var/tmp/*.tar"),"tar -tvf /var/tmp/*.tar"))
        command.append((_("tar -xvf /var/tmp/*.tar"),"tar -xvf /var/tmp/*.tar"))
        return command

###############################################################################
###############################################################################

class UserWizzard(Screen):
    def __init__(self, session):

        self.session = session
        self.session.openWithCallback(self.UserCommandSelected,ChoiceBox,_("select user command to execute"),self.getListUserCommands())        

    def UserCommandSelected(self,command):
        if command is None:
            self.skipUserselect("no user command selected - doing nothing")
        else:
            self.target = command[1]
            self.session.openWithCallback(self.setUserCommand,MessageBox,_("are you sure execute %s" % (self.target)), MessageBox.TYPE_YESNO)

    def setUserCommand(self,answer):
        if answer is None:
            self.skipUserselect("answer is None")
        if answer is False:
            self.skipUserselect("you were not confirming")
        else:
           self.session.open(Console,_("executing user command %s " % self.target),["%s/%s %s" % (shell_pluginpath,shell_script,self.target)])

    def skipUserselect(self,reason):
        self.session.open(MessageBox,_("selecting a user command was cancelled, because %s" % reason), MessageBox.TYPE_WARNING)

    def getListUserCommands(self):
        command = []
        commandfile = "%s/%s" % (shell_pluginpath,shell_commands)
        fp = open(commandfile,"r")

        for x in range(1,15):
            try:
                line =fp.readline()
                print "[SHELL] command %i is %s" % (x,line)
                command.append((_(line),"%s" %line))
            except IOError:
                print "[SHELL] no command %i" % x
                command.append((_(line),"#"))

        fp.close()
        return command

###############################################################################
###############################################################################

class KeyWizzard(Screen):
    def __init__(self, session):

        self.session = session
        self.session.openWithCallback(self.KeySelected,ChoiceBox,_("select an alternate keymap"),self.getListKeys())        

    def KeySelected(self,command):
        if command is None:
            self.skipKeyselect("no alternate keymap selected - doing nothing")
        else:
            self.target = command[1]
            self.session.openWithCallback(self.setKey,MessageBox,_("are you sure to load keymap %s" % (self.target)), MessageBox.TYPE_YESNO)

    def setKey(self,answer):
        if answer is None:
            self.skipKeyselect("answer is None")
        if answer is False:
            self.skipKeyselect("you were not confirming")
        else:
           self.session.open(Console,_("Loading alternate keymap %s " % self.target),["%s/%s alternatekeys %s" % (shell_pluginpath,shell_script,self.target)])

    def skipKeyselect(self,reason):
        self.session.open(MessageBox,_("selecting an alternate keymap was cancelled, because %s" % reason), MessageBox.TYPE_WARNING)

    def getListKeys(self):
        keymaps = []
        keymapfile = "%s/%s" % (shell_pluginpath,shell_keymaps)
        os.system("ls %s/keymaps > %s/%s" % (shell_pluginpath,shell_pluginpath,shell_keymaps))
        for x in range(1,15):
            os.system("echo \" \" >> %s/%s" % (shell_pluginpath,shell_keymaps))
        fp = open(keymapfile,"r")

        for x in range(1,15):
            try:
                line =fp.readline()
                print "[SHELL] keymap %i is %s" % (x,line)
                firstchar = line[0]
                if firstchar is "k": 
                   keymaps.append((_(line),"%s" %line))
            except IOError:
                print "[SHELL] no keymap %i" % x

        fp.close()
        return keymaps 

###############################################################################

